@@ -15,9 +15,10 @@ exception Parse_error of string * int * string
 
 val load : ?file : string -> unit -> unit Lwt.t
   (** [load ?file ()] loads key bindings from [file], which defaults
-      to ~/.config/.lambda-term-inputrc, if it exists. *)
+      to ~/.config/lambda-term-inputrc, if it exists. *)
 
 val default : string
   (** The name of the default key bindings file,
-      i.e. ~/.config/.lambda-term-inputrc
-      or the legacy location ~/.lambda-term-inputrc, if it exists *)
+      i.e. ~/.config/lambda-term-inputrc
+      or one of the legacy locations ~/.config/.lambda-term-inputrc
+      or ~/.lambda-term-inputrc, if they exist *)
